/* Copyright 2011 Joachim Zobel <jz-2011@heute-morgen.de>.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// $Id: mod_xml2.h 97 2012-04-15 18:22:52Z jo $

/**
 * mod_xml2 API
 */

#ifndef MOD_XML2_H
#define MOD_XML2_H

#include <http_protocol.h>
#include <http_log.h>

#include <apr_buckets.h>
#include <util_filter.h>
/* This does intentionally not include apr_optional.h. 
   You may want to use mod_xml2_extern.h if you want to use 
   mod_xml2 from another module. */
//#include <apr_optional.h>

#include <libxml/tree.h>

/**************************************************************************
 * Basic API
 **************************************************************************/

/**
 * These are the basic functions that are exported to high level languages.
 * They are (with the exception of mod_xml2_abort) implemented in the 
 * API section of node_buckets.c.
 */

/**
 * Retrieve the libxml2 node held by the bucket.
 * @param b the bucket
 * @return The libxml2 node or null if the bucket 
 *         is not of type NODE.
 */
APR_DECLARE_OPTIONAL_FN(xmlNode *, xml2_unwrap_node, (apr_bucket * b));

/**
 * Create a NODE bucket wrapping the given node.
 * @param a The allocator used to allocate the bucket memory
 * @param node The node that will be wrapped
 * @param r_log request object that will be used by the bucket for logging
 * @return The bucket.
 */
APR_DECLARE_OPTIONAL_FN(apr_bucket *, xml2_wrap_node,
                        (apr_bucket_alloc_t * a, xmlNode * node,
                         request_rec * r_log));


/**
 * Returns the bucket for the end element if and only if the 
 * bucket holds a start element and NULL otherwise.
 * @param b The bucket
 * @return The associated end bucket.
 */
APR_DECLARE_OPTIONAL_FN(apr_bucket *, xml2_end_bucket, (apr_bucket * b));

/**
 * Turns a bucket holding an element node into a start bucket 
 * and returns the newly created end bucket.  
 * @param b The element bucket
 * @return The associated end bucket on success, NULL otherwise.
 */
APR_DECLARE_OPTIONAL_FN(apr_bucket *, xml2_make_start_bucket,
                        (apr_bucket * b));

/** 
 * Aborts parsing end ends processing.
 * @param r - All NODE filters for this request are aborted.
 */
APR_DECLARE_OPTIONAL_FN(void, xml2_abort, (request_rec * r));

/**
 * A hook that is called when the document node has been created.
 */
APR_DECLARE_EXTERNAL_HOOK(xml2, AP, int, document_start, (request_rec *r, xmlDoc *doc));

/**************************************************************************
 * Simple XPath callback API
 **************************************************************************/
/**
 * This file implements callback functions for simple xpath expressions.
 * Every time the xpath is matched the callback gets called passing a 
 * document fragment holding the matched subtree.
 * To use these you need to register a filter that uses xml2_xpath_filter
 * and wraps xml2_xpath_filter_init in its init function. 
 * This filter processes node buckets produced by the mod_xml2 node filter.
 */

// Separate "only once", since this file is included via
// mod_xml2_extern.h _and_ mod_xml2_intern.h.
#ifndef _TREE_TRANSFORM
#define _TREE_TRANSFORM
/**
 * The XPath callback that manipulates a matched subtree.
 * @param node The document fragment holding the tree that shall 
 * be modified. Using a fragment allows removing the whole subtree.
 * @param param Additional parameters
 * @return APR_SUCCESS, if sucessful 
 */
typedef int (*tree_transform) (xmlNode * node, void *param);
#endif

/**
 * This does a full filter init for an XPath callback filter. It is 
 * however not a filter init function, as it requires the xpath as 
 * an additional parameter. 
 * @param f The filter
 * @param pattern The simple XPath pattern that triggers the callback
 * @param namespaces The namespaces needed to compile the pattern passed as 
 *                   [URI, prefix, ..., NULL, NULL] 
 *                   Be aware that these prefixes are just aliases for
 *                   pattern usage. They do not need to coincide with 
 *                   the prefixes in the document.
 * @param transform The XPath callback
 * @param param Additional data for the callback.
 * @return APR_SUCCESS, if sucessful 
 */
APR_DECLARE_OPTIONAL_FN(int, xml2_xpath_filter_init,
                        (ap_filter_t * f, const char *pattern,
                        const char **namespaces,
                        tree_transform transform, void *param));

/**
 * This does the simple XPath filtering. In case of a match the
 * subsequent matched buckets containing a subtree of the document
 * tree are passed to the next filter as a single brigade. 
 * Brigades of non matching buckets are passed to the second next
 * filter.
 * @param The filter
 * @param bb The bucket brigade
 * @return APR_SUCCESS, if sucessful  
 */
APR_DECLARE_OPTIONAL_FN(int, xml2_xpath_filter,
                        (ap_filter_t * f, apr_bucket_brigade * bb));

/**
 * Flattens a tree into a brigade
 * @param bb An empty brigade
 * @param tree an xmlNode and its children
 * @param r_log used for logging 
 */
APR_DECLARE_OPTIONAL_FN(void, xml2_tree_to_brigade,
                        (apr_bucket_brigade * bb, xmlNode * tree,
                         request_rec * r_log));

/**
 * Rebuilds a tree from a brigade
 * @param bb The brigade. It must start with a start bucket and 
 *         end with the corresponding end bucket.
 * @return The tree
 */
APR_DECLARE_OPTIONAL_FN(xmlNode *, xml2_tree_from_brigade,
                        (apr_bucket_brigade * bb));


/**
 * The size in pointer sizes reserved for the modperl_filter_ctx_t.
 * INTERNAL. Do not use.
 */
#define SIZE_modperl_filter_ctx_t (4)

#endif
